package IOTNetworkCard

import (
	"ERP_storage/models/Account"
	"encoding/json"
	orm2 "github.com/beego/beego/v2/client/orm"
	"gogs.baozhida.cn/zoie/ERP_libs/lib"
	"time"

	"ERP_storage/logs"
	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	_ "github.com/go-sql-driver/mysql"
)

const (
	Delete                  int = iota
	IOTNetworkCardNewCardIn     // 新卡入库
	IOTNetworkCardReturnIn      // 退还入库
	IOTNetworkCardOut           // 已出库
	IOTNetworkCardDamage        // 损坏退还
	IOTNetworkCardWaitAnnul     // 待注销
	IOTNetworkCardAnnul         // 已注销

)

var (
	IOTNetworkCardStateMap = map[int]string{
		IOTNetworkCardNewCardIn: "新卡入库",
		IOTNetworkCardReturnIn:  "退还入库",
		IOTNetworkCardOut:       "已出库",
		IOTNetworkCardDamage:    "损坏退还",
		IOTNetworkCardWaitAnnul: "待注销",
		IOTNetworkCardAnnul:     "已注销",
	}
)

// 物联网卡
type IOTNetworkCard struct {
	Id              int       `orm:"column(ID);size(11);auto;pk"`
	T_company_name  string    `orm:"size(256);null"`                                        // 入库日期
	T_in_date       string    `orm:"size(256);null"`                                        // 入库日期
	T_out_date      string    `orm:"size(256);null"`                                        // 出库日期
	T_meal_type     string    `orm:"size(256);null"`                                        // 套餐类型
	T_meal          string    `orm:"size(256);null"`                                        // 套餐类型
	T_expire        string    `orm:"size(256);null"`                                        // 到期&欠费
	T_iccid         string    `orm:"size(256);null"`                                        // 串码
	T_device_number string    `orm:"size(256);null"`                                        // 移动端设备编号
	T_MSISDN        string    `orm:"size(256);null;column(t_msisdn);"`                      // MSISDN
	T_out_project   string    `orm:"size(256);null"`                                        // 出库项目
	T_record        string    `orm:"type(text);null"`                                       // 出入库记录
	T_remark        string    `orm:"type(text);null"`                                       // 备注
	T_receive       string    `orm:"size(256);null"`                                        // 领取人
	T_State         int       `orm:"size(2);default(1);column(t_state);"`                   // 0 删除（伪删除）  1-新卡入库 2-退还入库 3-已出库 4-损坏退还 5-待注销 6-已注销
	CreateTime      time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime      time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

type IOTNetworkCardRecord struct {
	T_data        string //日期
	T_State       int
	T_State_name  string
	T_out_project string
	T_remark      string
}

func (t *IOTNetworkCard) TableName() string {
	return "iot_network_card" // 数据库名称   // ************** 替换 FormulaList **************
}

type IOTNetworkCardDaoImpl struct {
	orm orm.Ormer
}

func NewIOTNetworkCard(orm orm.Ormer) *IOTNetworkCardDaoImpl {
	return &IOTNetworkCardDaoImpl{orm: orm}
}

func init() {
	//注册模型
	orm.RegisterModel(new(IOTNetworkCard))
}

type IOTNetworkCard_R struct {
	Id              int
	T_company_name  string                 // 公司名称
	T_in_date       string                 // 入库日期
	T_out_date      string                 // 出库日期
	T_meal_type     string                 // 套餐类型
	T_meal          string                 // 套餐
	T_expire        string                 // 到期&欠费
	T_iccid         string                 // 串码
	T_device_number string                 // 移动端设备编号
	T_MSISDN        string                 // MSISDN
	T_out_project   string                 // 出库项目
	T_record        []IOTNetworkCardRecord // 出入库记录
	T_State         int
	T_State_name    string
	T_receive       string
	T_receive_name  string
	T_remark        string
}

func IOTNetworkCardToIOTNetworkCard_R(t IOTNetworkCard) (r IOTNetworkCard_R) {
	r.Id = t.Id
	r.T_company_name = t.T_company_name
	r.T_in_date = t.T_in_date
	r.T_out_date = t.T_out_date
	r.T_meal_type = t.T_meal_type
	r.T_meal = t.T_meal
	r.T_expire = t.T_expire
	r.T_iccid = t.T_iccid
	r.T_device_number = t.T_device_number
	r.T_MSISDN = t.T_MSISDN
	r.T_out_project = t.T_out_project
	var list []IOTNetworkCardRecord
	if len(t.T_record) > 0 {
		err := json.Unmarshal([]byte(t.T_record), &list)
		if err != nil {
			logs.Error("JSON 反序列化失败:", err)
		}
	}
	for i := 0; i < len(list); i++ {
		list[i].T_State_name = IOTNetworkCardStateMap[list[i].T_State]
	}
	r.T_record = list
	r.T_State = t.T_State
	r.T_State_name = IOTNetworkCardStateMap[t.T_State]
	r.T_receive = t.T_receive
	r.T_remark = t.T_remark
	r.T_receive_name = Account.Read_User_T_name_Get(t.T_receive)
	return r
}

// 添加
func (dao *IOTNetworkCardDaoImpl) Add_IOTNetworkCard(r IOTNetworkCard) (id int64, err error) {
	id, err = dao.orm.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return id, err
}

// 添加
func (dao *IOTNetworkCardDaoImpl) Add_IOTNetworkCardMulti(r []IOTNetworkCard) (id int64, err error) {
	id, err = dao.orm.InsertMulti(len(r), &r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return id, err
}

// 删除的重复数据
func (dao *IOTNetworkCardDaoImpl) DeleteDeduplicate() (int64, error) {
	tb_name := "iot_network_card"
	var cnt int64
	// 创建临时表
	sqlCreate := "CREATE TABLE `iot_network_card_tmp_table` AS (SELECT MIN(`ID`) AS `min_id` FROM " + tb_name + " GROUP BY `t_msisdn`);"
	_, err := dao.orm.Raw(sqlCreate).Exec()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return cnt, err
	}
	sqlDelete := "DELETE FROM " + tb_name + "  WHERE `ID` NOT IN (SELECT `min_id` FROM `iot_network_card_tmp_table`);"
	res, err := dao.orm.Raw(sqlDelete).Exec()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return cnt, err
	}
	cnt, _ = res.RowsAffected()
	sqlDrop := "DROP TABLE `iot_network_card_tmp_table`;"
	_, err = dao.orm.Raw(sqlDrop).Exec()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return cnt, err
	}
	return cnt, nil
}

// 获取 ById
func Read_IOTNetworkCard_ByT_iccid(T_iccid string) (r IOTNetworkCard, err error) {
	o := orm.NewOrm()
	qs := o.QueryTable(new(IOTNetworkCard))

	err = qs.Filter("T_iccid", T_iccid).Filter("T_State__gt", 0).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

func Read_IOTNetworkCard_ById(Id int) (r IOTNetworkCard, err error) {
	o := orm.NewOrm()
	qs := o.QueryTable(new(IOTNetworkCard))
	err = qs.Filter("Id", Id).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

// 修改
func (dao *IOTNetworkCardDaoImpl) Update_IOTNetworkCard(m IOTNetworkCard, cols ...string) error {
	_, err := dao.orm.Update(&m, cols...)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return err
	}
	return nil
}

// 删除
func (dao *IOTNetworkCardDaoImpl) Delete_IOTNetworkCard(v IOTNetworkCard) error {
	v.T_State = 0
	_, err := dao.orm.Update(&v, "T_State")
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return err
}

// 获取列表
func (dao *IOTNetworkCardDaoImpl) Read_IOTNetworkCard_List(T_name, T_company_name string, T_state, page, page_z int) (r_ []IOTNetworkCard_R, cnt int64) {
	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(IOTNetworkCard))
	var offset int64
	if page <= 1 {
		offset = 0
	} else {
		offset = int64((page - 1) * page_z)
	}

	// 过滤
	cond := orm.NewCondition()
	cond1 := cond.And("T_State__gt", 0)
	if T_state > 0 {
		cond1 = cond1.And("T_State", T_state)
	}
	if len(T_company_name) > 0 {
		cond1 = cond1.And("T_company_name__icontains", T_company_name)
	}
	if len(T_name) > 0 {
		cond1 = cond1.AndCond(cond.Or("T_iccid__icontains", T_name).Or("T_MSISDN__icontains", T_name).Or("T_out_project__icontains", T_name))
	}

	// 查询
	var r []IOTNetworkCard
	_, err := qs.Limit(page_z, offset).SetCond((*orm2.Condition)(cond1)).OrderBy("-Id").All(&r)

	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	cnt, err = qs.SetCond((*orm2.Condition)(cond1)).Count()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	for _, v := range r {
		r_ = append(r_, IOTNetworkCardToIOTNetworkCard_R(v))
	}

	return r_, cnt
}
