package Percentage

import (
	"ERP_storage/logs"
	"ERP_storage/models/Account"
	"fmt"
	orm2 "github.com/beego/beego/v2/client/orm"
	"gogs.baozhida.cn/zoie/ERP_libs/lib"
	"strconv"
	"strings"
	"time"

	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	_ "github.com/go-sql-driver/mysql"
)

const (
	Delete      int = iota
	WaitAudit       // 待审核
	AuditPass       // 审核通过
	AuditUnPass     // 审核不通过
	RemitPart       // 部分打款
	RemitAll        // 全部打款
	NotSubmit       // 未提交审核

)

var (
	AuditMap = map[int]string{
		WaitAudit:   "待审核",
		AuditPass:   "审核通过",
		AuditUnPass: "审核不通过",
		RemitPart:   "部分打款",
		RemitAll:    "全部打款",
		NotSubmit:   "未提交审核",
	}
	DeviceTypeMap = map[string]string{
		"X":  "箱",
		"G":  "柜",
		"C":  "车",
		"K":  "库",
		"XT": "系统",
		"WZ": "位置",
		"PX": "培训",
		"XJ": "巡检",
		"QT": "其他",
	}
	SchemeType    = 1
	ReportingType = 2
)

func Read_Audit_Get(Id int) string {
	v, ok := AuditMap[Id]
	if ok {
		return v
	} else {
		return "未知状态"
	}
}

// 提成
type Percentage struct {
	Id                    int     `orm:"column(ID);size(11);column(id);auto;pk"`
	T_Distributor_id      string  `orm:"size(256);column(t_distributor_id);null"` // 分销商id
	T_task_id             string  `orm:"size(256);null"`                          // 任务id
	T_task_name           string  `orm:"size(256);null"`                          // 任务名称
	T_task_int_id         int     `orm:"size(256);null"`                          // 任务id
	T_device_type         string  `orm:"size(256);null"`                          // 设备类型
	T_verify_type         string  `orm:"size(256);null"`                          // 验证类型
	T_device_quantity     int     `orm:"size(256);null"`                          // 终端数量
	T_company_uuid        string  `orm:"size(256);null"`                          // 提成公司uuid
	T_uuid                string  `orm:"size(256);null"`                          // 提成人员uuid
	T_money               float32 `orm:"digits(12);decimals(2)"`                  // 提成金额
	T_type                int     `orm:"size(2);default(1)"`                      // 类型 1-验证实施 2-报告编写
	T_State               int     `orm:"size(2);column(t_state);default(1)"`      // 0 删除（伪删除）  1-待审核 2-审核通过 3-审核不通过 4-部分打款 5-全部打款 6-未提交审核 审核状态
	T_verify_item         int     `orm:"size(2);default(1)"`                      // 提成项目id
	T_reporting_pass_time string  `orm:"size(256);null"`                          // 验证报告负责人通过时间
	T_remit               string  `orm:"type(text);null"`                         // 打款信息 打款时间,打款金额|打款时间,打款金额|
	T_approval_money      float32 `orm:"digits(12);decimals(2)"`                  // 审批金额
	T_approval_opinion    string  `orm:"type(text);null"`                         // 审批意见

	CreateTime time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

func (t *Percentage) TableName() string {
	return "percentage" // 数据库名称   // ************** 替换 FormulaList **************
}

type PercentageDaoImpl struct {
	orm orm.Ormer
}

func NewPercentage(orm orm.Ormer) *PercentageDaoImpl {
	return &PercentageDaoImpl{orm: orm}
}

func init() {
	//注册模型
	orm.RegisterModel(new(Percentage))
}

type Percentage_R struct {
	Id int

	T_Distributor_id      string
	T_task_id             string
	T_device_type         string
	T_verify_type         string
	T_device_quantity     int
	T_company_uuid        string
	T_task_name           string
	T_reporting_pass_time string

	T_uuid           string  // 提成人员uuid
	T_uuid_name      string  // 提成人员uuid
	T_money          float32 // 提成金额
	T_type           int     // 类型 1-验证实施 2-报告编写
	T_approval_money float32 // 审批金额
	T_send_money     float32 // 提成已打款金额
	T_State          int     // 0 删除（伪删除）  1-待审核 2-审核通过 3-审核不通过 4-部分打款 5-全部打款 6-未提交审核 审核状态
	T_verify_item    int
	T_verifyCompany  VerifyCompany_R // 验证公司信息

}

type Percentage_Detail struct {
	Id int

	T_Distributor_id      string
	T_task_id             string
	T_device_type         string
	T_verify_type         string
	T_company_uuid        string
	T_task_name           string
	T_reporting_pass_time string

	T_uuid             string              // 提成人员uuid
	T_uuid_name        string              // 提成人员uuid
	T_money            float32             // 提成金额
	T_approval_money   float32             // 审批金额
	T_approval_opinion string              // 审批意见
	T_State            int                 // 0 删除（伪删除）  1-待审核 2-审核通过 3-审核不通过 4-部分打款 5-全部打款 6-未提交审核 审核状态
	T_type             int                 // 类型 1-验证实施 2-报告编写
	T_remit            []PercentageRemit_R // 提成打款明细 打款时间,打款金额|打款时间,打款金额|
	T_send_money       float32             // 提成已打款金额
	T_verify_item      int

	T_verifyCompany VerifyCompany_R // 验证公司信息
}

// 提成打款明细
type PercentageRemit_R struct {
	T_date  string  // 时间
	T_money float32 // 金额）
}

// 提成打款明细
type PercentageItem_R struct {
	T_id    int     // 验证项目id
	T_name  string  // 名称
	T_num   int     // 数量
	T_price float32 //  金额
}

type VerifyTask struct {
	Id                int    `orm:"column(ID);size(11);auto;pk"`
	T_Distributor_id  string `orm:"size(256);null"`     // 分销商id
	T_task_id         string `orm:"size(256);null"`     // 任务ID
	T_uuid            string `orm:"size(256);null"`     // 用户 UUID
	T_name            string `orm:"size(256);null"`     // 标题
	T_scheme          string `orm:"size(256);null"`     // 实施方案 负责人UUID
	T_collection      string `orm:"size(256);null"`     // 数据采集 负责人UUID
	T_reporting       string `orm:"size(256);null"`     // 报告编写 负责人UUID
	T_delivery        string `orm:"size(256);null"`     // 交付审核 负责人UUID
	T_scheme_state    int    `orm:"size(2);default(0)"` // 实施方案 状态 0 未完成 1 已完成(客户通过) 2已退回(客户) 3已通过(负责人) 4已退回(负责人) 5已提交
	T_reporting_state int    `orm:"size(2);default(0)"` // 报告编写 状态 0 未完成 1 已完成(客户通过) 2已退回(客户) 3已通过(负责人) 4已退回(负责人) 5已提交
	T_State           int    `orm:"size(2);default(1)"` // 0 删除   1 正常

	T_device_type string `orm:"size(256);null"` // 设备类型

	T_verify_type         string `orm:"size(256);null"` // 验证类型
	T_reporting_pass_time string `orm:"size(256);null"` // 验证报告负责人通过时间

	T_device_quantity int `orm:"size(2);default(0)"` // 终端数量

	CreateTime time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

func PercentageToPercentageRemit_R(T_detail string) (r []PercentageRemit_R, money float32) {

	if len(T_detail) == 0 {
		return
	}
	detailList := strings.Split(strings.Trim(T_detail, "|"), "|")
	for _, detail := range detailList {
		T_date := strings.Split(detail, ",")[0]
		T_money := strings.Split(detail, ",")[1]
		temp := float32(lib.StringToFloat64TwoDecimal(T_money))
		money += temp
		r = append(r, PercentageRemit_R{
			T_date:  T_date,
			T_money: temp,
		})

	}
	return r, money
}

func PercentageToPercentage_R(t Percentage) (r Percentage_R) {
	r.Id = t.Id
	r.T_Distributor_id = t.T_Distributor_id
	r.T_task_id = t.T_task_id
	r.T_task_name = t.T_task_name
	r.T_device_type = t.T_device_type
	r.T_verify_type = t.T_verify_type
	r.T_device_quantity = t.T_device_quantity
	r.T_company_uuid = t.T_company_uuid
	if len(t.T_reporting_pass_time) > 0 {
		r.T_reporting_pass_time = t.T_reporting_pass_time[:10]
	}

	r.T_verify_item = t.T_verify_item
	r.T_verifyCompany = Read_VerifyCompany_Get(t.T_company_uuid)

	r.T_uuid = t.T_uuid
	r.T_uuid_name = Account.Read_User_T_name_Get(t.T_uuid)
	r.T_money = t.T_money
	r.T_approval_money = t.T_approval_money
	_, r.T_send_money = PercentageToPercentageRemit_R(t.T_remit)
	r.T_type = t.T_type
	r.T_State = t.T_State
	return r
}

func PercentageToPercentage_Detail(t Percentage) (r Percentage_Detail) {
	r.Id = t.Id
	r.T_Distributor_id = t.T_Distributor_id
	r.T_task_id = t.T_task_id
	r.T_task_name = t.T_task_name
	r.T_device_type = t.T_device_type
	r.T_verify_type = t.T_verify_type
	r.T_company_uuid = t.T_company_uuid
	if len(t.T_reporting_pass_time) > 0 {
		r.T_reporting_pass_time = t.T_reporting_pass_time[:10]
	}
	r.T_verify_item = t.T_verify_item
	r.T_verifyCompany = Read_VerifyCompany_Get(t.T_company_uuid)

	r.T_uuid = t.T_uuid
	r.T_uuid_name = Account.Read_User_T_name_Get(t.T_uuid)
	r.T_money = t.T_money
	r.T_approval_money = t.T_approval_money
	r.T_approval_opinion = t.T_approval_opinion
	r.T_remit, r.T_send_money = PercentageToPercentageRemit_R(t.T_remit)
	r.T_type = t.T_type
	r.T_State = t.T_State

	return r
}

// 添加
func (dao *PercentageDaoImpl) Add_Percentage(r Percentage) (id int64, err error) {
	id, err = dao.orm.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return id, err
}

// 获取 ById
func (dao *PercentageDaoImpl) Read_Percentage_ByT_T_task_id_T_type(T_task_id string, T_type int) (r Percentage, err error) {
	qs := dao.orm.QueryTable(new(Percentage))

	err = qs.Filter("T_task_id", T_task_id).Filter("T_type", T_type).Filter("T_State__gt", 0).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

func (dao *PercentageDaoImpl) Read_Percentage_ById(Id int) (r Percentage, err error) {
	qs := dao.orm.QueryTable(new(Percentage))
	err = qs.Filter("Id", Id).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

// 修改
func (dao *PercentageDaoImpl) Update_Percentage(m Percentage, cols ...string) error {
	_, err := dao.orm.Update(&m, cols...)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return err
	}
	return nil
}

// 删除
func (dao *PercentageDaoImpl) Delete_Percentage(v Percentage) error {
	v.T_State = 0
	_, err := dao.orm.Update(&v, "T_State")
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return err
}

// 获取列表
func (dao *PercentageDaoImpl) Read_Percentage_List(T_name, T_uuid, T_start_date, T_end_date string, T_company_list []string, T_state, T_type, page, page_z int) (r_ []Percentage_R, cnt int64) {
	// 也可以直接使用 Model 结构体作为表名
	qs := dao.orm.QueryTable(new(Percentage))
	var offset int64
	if page <= 1 {
		offset = 0
	} else {
		offset = int64((page - 1) * page_z)
	}

	// 过滤
	cond := orm.NewCondition()
	cond1 := cond.And("T_State__gt", 0)
	if T_state > 0 {
		cond1 = cond1.And("T_State", T_state)
	} else {
		cond1 = cond1.AndNot("T_State", 6)
	}

	if len(T_name) > 0 {
		cond1 = cond1.AndCond(cond1.Or("T_task_id__icontains", T_name).Or("T_company_uuid__in", T_company_list))
	}
	if len(T_uuid) > 0 {
		cond1 = cond1.And("T_uuid", T_uuid)
	}
	if T_type > 0 {
		cond1 = cond1.And("T_type", T_type)
	}
	if len(T_start_date) > 0 {
		cond1 = cond1.And("T_reporting_pass_time__gte", T_start_date+" 00:00:00")
	}

	if len(T_start_date) > 0 {
		cond1 = cond1.And("T_reporting_pass_time__lte", T_end_date+" 23:59:59")
	}
	// 查询
	var r []Percentage
	_, err := qs.Limit(page_z, offset).SetCond((*orm2.Condition)(cond1)).OrderBy("T_State", "-T_task_int_id").All(&r)

	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	cnt, err = qs.SetCond((*orm2.Condition)(cond1)).Count()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	for _, v := range r {
		r_ = append(r_, PercentageToPercentage_R(v))
	}

	return r_, cnt
}
func (dao *PercentageDaoImpl) Read_Percentage_List_SQL(T_name, T_uuid, T_start_date, T_end_date string, T_export bool, T_state, T_type, page, page_z int) (r_ []Percentage_R, cnt int64) {
	// 也可以直接使用 Model 结构体作为表名
	var offset int
	if page <= 1 {
		offset = 0
	} else {
		offset = (page - 1) * page_z
	}

	var maps_z []orm2.ParamsList
	var r []Percentage
	sqlWhere := " WHERE p.t_state > 0"
	if T_state > 0 {
		sqlWhere += " AND p.t_state = " + strconv.Itoa(T_state)
	} else {
		sqlWhere += " AND p.t_state != " + strconv.Itoa(6)
	}
	if len(T_name) > 0 {
		sqlWhere += " AND (p.t_task_id like \"%" + T_name + "%\" or vc.T_name like \"%" + T_name + "%\")"
	}
	if T_type > 0 {
		sqlWhere += " AND p.t_type != " + strconv.Itoa(T_type)

	}
	if len(T_uuid) > 0 {
		sqlWhere += " AND p.t_uuid = \"" + T_uuid + "\""
	}
	if len(T_start_date) > 0 {
		sqlWhere += " AND vc.t_recoveries_time >= \"" + T_start_date + "\""
	}
	if len(T_end_date) > 0 {
		sqlWhere += " AND vc.t_recoveries_time <= \"" + T_end_date + "\""
	}

	// 获取总条数
	sql := "SELECT COUNT(p.ID) FROM percentage p " +
		"LEFT JOIN verify_company vc ON p.t_company_uuid=vc.t_uuid "
	sql = sql + sqlWhere
	fmt.Println(sql)
	_, err := dao.orm.Raw(sql).ValuesList(&maps_z)
	if err != nil {
		return r_, 0
	}
	if len(maps_z) == 0 {
		return r_, 0
	}
	sql = "SELECT p.t_state as t_state,p.* FROM percentage p " +
		"LEFT JOIN verify_company vc ON p.t_company_uuid=vc.t_uuid"

	sql = sql + sqlWhere

	if T_export {
		sql += " ORDER BY t_company_uuid asc,p.t_task_int_id asc"
	} else {
		sql += " ORDER BY p.t_task_int_id DESC"
	}

	if page_z != 9999 {
		sql += " LIMIT " + strconv.Itoa(offset) + "," + strconv.Itoa(page_z)
	}

	fmt.Println(sql)
	_, err = dao.orm.Raw(sql).QueryRows(&r)
	for _, v := range r {
		r_ = append(r_, PercentageToPercentage_R(v))
	}

	count, _ := strconv.Atoi(maps_z[0][0].(string))
	return r_, int64(count)
}
func (dao *PercentageDaoImpl) Read_Percentage_User_List_SQL(T_uuid, T_name, T_start_date, T_end_date string, T_export bool, T_state, page, page_z int) (r_ []Percentage_R, cnt int64) {
	// 也可以直接使用 Model 结构体作为表名
	var offset int
	if page <= 1 {
		offset = 0
	} else {
		offset = (page - 1) * page_z
	}

	var maps_z []orm2.ParamsList
	var r []Percentage
	sqlWhere := " WHERE p.t_state > 0"
	if T_state > 0 {
		sqlWhere += " AND p.t_state = " + strconv.Itoa(T_state)
	}
	if len(T_name) > 0 {
		sqlWhere += " AND (p.t_task_id like \"%" + T_name + "%\" or vc.T_name like \"%" + T_name + "%\")"
	}
	if len(T_uuid) > 0 {
		sqlWhere += " AND p.t_uuid = \"" + T_uuid + "\""
	}
	if len(T_start_date) > 0 {
		sqlWhere += " AND vc.t_recoveries_time >= \"" + T_start_date + "\""
	}
	if len(T_end_date) > 0 {
		sqlWhere += " AND vc.t_recoveries_time <= \"" + T_end_date + "\""
	}

	// 获取总条数
	sql := "SELECT COUNT(p.ID) FROM percentage p " +
		"LEFT JOIN verify_company vc ON p.t_company_uuid=vc.t_uuid "
	sql = sql + sqlWhere
	fmt.Println(sql)
	_, err := dao.orm.Raw(sql).ValuesList(&maps_z)
	if err != nil {
		return r_, 0
	}
	if len(maps_z) == 0 {
		return r_, 0
	}
	sql = "SELECT p.t_state as t_state,p.* FROM percentage p " +
		"LEFT JOIN verify_company vc ON p.t_company_uuid=vc.t_uuid"

	sql = sql + sqlWhere

	if T_export {
		sql += " ORDER BY t_company_uuid asc,p.t_task_int_id asc"
	} else {
		sql += " ORDER BY p.t_task_int_id DESC"
	}

	if page_z != 9999 {
		sql += " LIMIT " + strconv.Itoa(offset) + "," + strconv.Itoa(page_z)
	}

	fmt.Println(sql)
	_, err = dao.orm.Raw(sql).QueryRows(&r)
	for _, v := range r {
		r_ = append(r_, PercentageToPercentage_R(v))
	}

	count, _ := strconv.Atoi(maps_z[0][0].(string))
	return r_, int64(count)
}

func (dao *PercentageDaoImpl) Read_T_uuid_List() (lists []string) {

	o := orm.NewOrm()
	sql := "SELECT DISTINCT t_uuid FROM percentage WHERE t_state > 0 LIMIT 0,1000"
	_, err := o.Raw(sql).QueryRows(&lists)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}
	return lists
}
