package Percentage

import (
	"ERP_storage/logs"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	_ "github.com/go-sql-driver/mysql"
	uuid "github.com/satori/go.uuid"
	"gogs.baozhida.cn/zoie/ERP_libs/lib"
	"log"
	"sync"
	"time"
)

var VerifyCompanyMap *sync.Map // 泛型

type VerifyCompany struct {
	Id                 int       `orm:"column(ID);size(11);auto;pk"`
	T_Distributor_id   string    `orm:"size(256);column(t_distributor_id);null"`               // 分销商id
	T_Distributor_name string    `orm:"size(256);column(t_distributor_name);null"`             // 分销商id
	T_uuid             string    `orm:"size(256);null"`                                        //
	T_name             string    `orm:"size(256);null"`                                        // 某某公司名称
	T_recoveries_time  string    `orm:"size(200);null"`                                        // 回款时间
	T_State            int       `orm:"size(200);column(t_state);default(1)"`                  // 0删除  1未回款 2已回款
	CreateTime         time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now 每次 model 保存时都会对时间自动更新
	UpdateTime         time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now_add 第一次保存时才设置时间
}
type VerifyCompany_R struct {
	T_uuid             string
	T_name             string
	T_Distributor_id   string
	T_Distributor_name string
	T_recoveries_time  string
	T_State            int
}

func (t *VerifyCompany) TableName() string {
	return "verify_company" // 数据库名称   // ************** 替换 FormulaList **************
}

func init() {
	//注册模型
	orm.RegisterModel(new(VerifyCompany))
	VerifyCompanyMap = new(sync.Map)

}

func Read_VerifyCompany_All_Map() {
	var list []VerifyCompany
	o := orm.NewOrm()
	qs := o.QueryTable(new(VerifyCompany))
	_, err := qs.Filter("T_State__gt", 0).All(&list)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	for _, v := range list {
		VerifyCompanyMap.Store(v.T_uuid, VerifyCompanyToVerifyCompany_R(v))
	}
}
func Read_VerifyCompany_Get(T_uuid string) VerifyCompany_R {
	v, ok := VerifyCompanyMap.Load(T_uuid)
	if ok {
		return v.(VerifyCompany_R)
	} else {
		return VerifyCompany_R{}
	}
}

// -------------------------------------------------------------
func VerifyCompanyToVerifyCompany_R(T VerifyCompany) (T_r VerifyCompany_R) {
	T_r.T_uuid = T.T_uuid
	T_r.T_Distributor_id = T.T_Distributor_id
	T_r.T_Distributor_name = T.T_Distributor_name
	T_r.T_recoveries_time = T.T_recoveries_time
	T_r.T_name = T.T_name
	T_r.T_State = T.T_State
	return T_r
}

// 获取 By
func Read_VerifyCompany(T_uuid string) (r VerifyCompany, is bool) {

	o := orm.NewOrm()
	qs := o.QueryTable(new(VerifyCompany))
	err := qs.Filter("T_uuid", T_uuid).Filter("T_State__gt", 0).One(&r)
	if err != nil {
		return r, false
	}

	return r, true
}

// 添加
func Add_VerifyCompany(r VerifyCompany) (id int64, is bool) {
	o := orm.NewOrm()
	// 生成编号
	rand_x := 0
	if len(r.T_uuid) == 0 {
		for true {
			r.T_uuid = uuid.NewV4().String()
			err := o.Read(&r, "T_uuid") // o.Read(&r,"Tokey") 如果不是 主键 就得指定字段名
			if err != nil {
				break
			}
			rand_x += 1
		}
	}

	id, err := o.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return 0, false
	}

	return id, true
}

// 删除
func Delete_VerifyCompany(v VerifyCompany) bool {
	o := orm.NewOrm()
	v.T_State = 0
	if num, err := o.Update(&v, "T_State"); err == nil {
		log.Println("Number of records updated in database:", num)
	} else {
		return false
	}

	return true
}

// 修改
func Update_VerifyCompany(m VerifyCompany, cols ...string) bool {
	o := orm.NewOrm()
	if num, err := o.Update(&m, cols...); err == nil {
		log.Println("Number of records updated in database:", num)

		return true
	}
	return false
}

// 获取列表
func Read_VerifyCompany_List(T_name string, T_State int, page int, page_z int) ([]VerifyCompany_R, int64) {
	o := orm.NewOrm()

	// 也可以直接使用 Model 结构体作为表名
	var r []VerifyCompany
	qs := o.QueryTable(new(VerifyCompany))
	var offset int64
	if page <= 1 {
		offset = 0
	} else {
		offset = int64((page - 1) * page_z)
	}
	cond := orm.NewCondition()
	cond1 := cond.And("T_State__gt", 0).AndCond(cond.Or("T_name__icontains", T_name))

	if T_State > 0 {
		cond1 = cond.And("T_State", T_State)
	}
	if len(T_name) > 0 {
		cond1 = cond1.AndCond(cond.Or("T_name__icontains", T_name).Or("T_Distributor_name__icontains", T_name))
	}
	qs.Limit(page_z, offset).SetCond((*orm2.Condition)(cond1)).OrderBy("-Id").All(&r)
	cnt, _ := qs.SetCond((*orm2.Condition)(cond1)).Count()

	// 转换
	var VerifyCompany_r []VerifyCompany_R
	for _, v := range r {
		VerifyCompany_r = append(VerifyCompany_r, VerifyCompanyToVerifyCompany_R(v))
	}

	return VerifyCompany_r, cnt
}

func Read_VerifyCompany_T_uuid_ListByT_name(T_name string) (list []string) {
	o := orm.NewOrm()

	// 也可以直接使用 Model 结构体作为表名
	var r []VerifyCompany
	qs := o.QueryTable(new(VerifyCompany))

	cond := orm.NewCondition()
	cond1 := cond.And("T_State__gt", 0).AndCond(cond.Or("T_name__icontains", T_name))
	qs.SetCond((*orm2.Condition)(cond1)).All(&r)

	// 转换
	for _, v := range r {
		list = append(list, v.T_uuid)
	}

	return list
}

// 获取 By T_name
func Read_VerifyCompanyByT_name(T_name string) (r VerifyCompany, is bool) {

	o := orm.NewOrm()
	qs := o.QueryTable(new(VerifyCompany))
	err := qs.Filter("T_name", T_name).Filter("T_State__gt", 0).One(&r)
	if err != nil {
		return r, false
	}

	return r, true
}
