package Purchase

import (
	"ERP_storage/logs"
	"ERP_storage/models/Account"
	"fmt"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	"gogs.baozhida.cn/zoie/ERP_libs/lib"
	"strings"
	"time"
)

const (
	Delete            = iota
	WaitPurchase      // 待审核
	PurchaseCompleted // 进行中

)

// 项目管理
type Purchase struct {
	Id              int    `orm:"column(ID);size(11);auto;pk"`
	T_uuid          string `orm:"index;size(32);null"` // 申请人
	T_submit        string `orm:"index;size(32);null"` // 提交人
	T_date          string `orm:"size(256);null"`      // 申请时间
	T_detail        string `orm:"type(text);null"`     // 明细
	T_remark        string `orm:"type(text);null"`     // 备注
	T_approver      string `orm:"size(32);null"`       // 审批人
	T_approver_date string `orm:"size(32);null"`       // 审批时间

	T_State    int       `orm:"size(2);column(t_state);default(1)"`                    // 0 删除  1 待采购  2 已采购
	CreateTime time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

func (t *Purchase) TableName() string {
	return "purchase"
}

type PurchaseDaoImpl struct {
	orm orm.Ormer
}

func NewPurchase(orm orm.Ormer) *PurchaseDaoImpl {
	return &PurchaseDaoImpl{orm: orm}
}
func init() {
	//注册模型
	orm2.Debug = true
	orm.RegisterModel(new(Purchase))
}

type Purchase_R struct {
	Id              int
	T_uuid          string // 申请人
	T_uuid_name     string // 申请人名称
	T_submit        string // 提交人
	T_submit_name   string // 提交人
	T_date          string // 申请时间
	T_remark        string // 备注
	T_approver      string // 审批人
	T_approver_name string // 审批人
	T_approver_date string // 审批时间
	T_State         int
}

type PurchaseDetail_R struct {
	T_name           string // 名称
	T_model          string // 型号
	T_spec           string // 规格
	T_quantity       int    // 数量
	T_demand         string // 需求
	T_remark         string // 备注
	T_state          int    // 状态
	T_date           string // 采购时间
	T_unit_price     string // 采购单价
	T_amount         string // 采购金额
	T_reference_site string // 参考网址
}
type Purchase_Detail struct {
	Id              int
	T_uuid          string // 申请人
	T_uuid_name     string // 用户名称
	T_submit        string // 提交人
	T_submit_name   string // 提交人
	T_date          string // 申请时间
	T_remark        string // 备注
	T_approver      string // 审批人
	T_approver_name string // 审批人
	T_approver_date string // 审批时间
	T_State         int
	T_Detail        []PurchaseDetail_R // 项目明细
}

// ---------------- 特殊方法 -------------------
func PurchaseToPurchase_R(t Purchase) (r Purchase_R) {

	r.Id = t.Id
	r.T_uuid = t.T_uuid
	r.T_uuid_name = Account.Read_User_T_name_Get(t.T_uuid)
	r.T_submit = t.T_submit
	r.T_submit_name = Account.Read_User_T_name_Get(t.T_submit)
	r.T_date = t.T_date
	r.T_remark = t.T_remark
	r.T_approver = t.T_approver
	r.T_approver_name = Account.Read_User_T_name_Get(t.T_approver)
	r.T_approver_date = t.T_approver_date
	r.T_State = t.T_State

	return r
}
func PurchaseToPurchase_Detail(t Purchase) (r Purchase_Detail) {

	r.Id = t.Id
	r.T_uuid = t.T_uuid
	r.T_uuid_name = Account.Read_User_T_name_Get(t.T_uuid)
	r.T_submit = t.T_submit
	r.T_submit_name = Account.Read_User_T_name_Get(t.T_submit)
	r.T_date = t.T_date
	r.T_remark = t.T_remark
	r.T_approver = t.T_approver
	r.T_approver_name = Account.Read_User_T_name_Get(t.T_approver)
	r.T_approver_date = t.T_approver_date
	r.T_State = t.T_State
	r.T_Detail = PurchaseToPurchaseDetail(t.T_detail)
	return r
}

func PurchaseToPurchaseDetail(T_detail string) (r []PurchaseDetail_R) {

	if len(T_detail) == 0 {
		return
	}

	detailList := strings.Split(strings.Trim(T_detail, "|"), "|")

	for _, detail := range detailList {
		T_name := strings.Split(detail, ",")[0]
		T_model := strings.Split(detail, ",")[1]
		T_spec := strings.Split(detail, ",")[2]
		T_quantity := strings.Split(detail, ",")[3]
		T_demand := strings.Split(detail, ",")[4]
		T_remark := strings.Split(detail, ",")[5]
		T_state := strings.Split(detail, ",")[6]
		T_date := strings.Split(detail, ",")[7]
		T_unit_price := strings.Split(detail, ",")[8]
		T_amount := strings.Split(detail, ",")[9]
		T_reference_site := strings.Split(detail, ",")[10]

		r = append(r, PurchaseDetail_R{
			T_name:           T_name,
			T_model:          T_model,
			T_spec:           T_spec,
			T_quantity:       lib.To_int(T_quantity),
			T_demand:         T_demand,
			T_remark:         T_remark,
			T_state:          lib.To_int(T_state),
			T_date:           T_date,
			T_unit_price:     T_unit_price,
			T_amount:         T_amount,
			T_reference_site: T_reference_site,
		})
	}
	return r
}

func (dao *PurchaseDaoImpl) Read_Purchase_ById(id int) (r Purchase, err error) {
	o := orm.NewOrm()
	r = Purchase{Id: id}
	err = o.Read(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return r, err
	}
	return r, err
}

// 添加
func (dao *PurchaseDaoImpl) Add_Purchase(m Purchase) (id int64, err error) {
	o := orm.NewOrm()
	id, err = o.Insert(&m)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}
	return
}

// 修改
func (dao *PurchaseDaoImpl) Update_Purchase(m Purchase, cols ...string) (id int64, err error) {
	o := orm.NewOrm()
	id, err = o.Update(&m, cols...)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}
	fmt.Println("Number of records updated in database:", id)
	return id, nil
}

// 删除
func (dao *PurchaseDaoImpl) Delete_Purchase(m Purchase) (id int64, err error) {

	o := orm.NewOrm()
	m.T_State = Delete
	id, err = o.Update(&m, "T_State")
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}
	fmt.Println("Number of records updated in database:", id)
	return id, nil
}

// 获取列表
func (dao *PurchaseDaoImpl) Read_Purchase_List(T_uid, T_approver string, T_state int, T_start_date, T_end_date string, page, page_z int) (r []Purchase_R, cnt int64) {

	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名
	var map_r []Purchase
	qs := o.QueryTable(new(Purchase))
	var offset int64
	if page <= 1 {
		offset = 0
	} else {
		offset = int64((page - 1) * page_z)
	}

	cond := orm.NewCondition()
	cond = cond.And("T_State__gt", Delete)
	if len(T_uid) > 0 {
		cond = cond.AndCond(cond.Or("T_uuid", T_uid).Or("T_submit", T_uid))
	}
	// 审批人
	if len(T_approver) > 0 {
		cond = cond.And("T_approver", T_approver)
	}
	if T_state > 0 {
		cond = cond.And("T_state", T_state)
	}
	if len(T_start_date) > 0 {
		cond = cond.And("T_date__gte", T_start_date)
	}

	if len(T_start_date) > 0 {
		cond = cond.And("T_date__lte", T_end_date)
	}
	var err error
	if page_z == 9999 {
		_, err = qs.SetCond((*orm2.Condition)(cond)).OrderBy("-Id").All(&map_r)
	} else {
		_, err = qs.Limit(page_z, offset).SetCond((*orm2.Condition)(cond)).OrderBy("-Id").All(&map_r)
	}

	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}
	cnt, err = qs.SetCond((*orm2.Condition)(cond)).Count()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	for _, v := range map_r {
		r = append(r, PurchaseToPurchase_R(v))
	}

	return r, cnt

}
