package validationtool

import (
	"ERP_storage/logs"
	"fmt"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	"gogs.baozhida.cn/zoie/ERP_libs/lib"
	"strconv"
	"time"
)

type ValidationToolHistory struct {
	Id               int       `orm:"column(ID);size(11);auto;pk"`                           // 合同编号
	T_class          int       `orm:"size(20);default(0)"`                                   // 产品分类 T_class=0 冷链验证 	// 产品id
	Validationnumber string    `orm:"size(256);null"`                                        // 验证工具编号
	T_sn             string    `orm:"size(256);null"`                                        // 设备sn
	T_iccid          string    `orm:"size(256);null"`                                        // sim卡号
	T_imei           string    `orm:"size(256);null"`                                        // 模组imei
	History_iccid    string    `orm:"size(256);type(json);null"`                             // 历史sim卡号
	History_imei     string    `orm:"size(256);type(json);null"`                             // 历史模组
	T_state          int       `orm:"size(2);default(2)"`                                    // 1-已出库 2-待使用(已入库)  3-维修中 4-已报废
	T_remark         string    `orm:"type(text);null"`                                       // 备注r
	T_project        string    `orm:"type(text);null"`                                       // 出库项目
	T_uuid           string    `orm:"size(256);null"`                                        //用户uuid
	LendUser         string    `orm:"size(256);null"`                                        //借出用户
	BatchNumber      string    `orm:"size(256);null"`                                        //批号
	CreateTime       time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime       time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

type ValidationToolHistory_R struct {
	Id               int    // 合同编号
	T_class          int    // 产品分类 T_class=0 冷链验证 	// 产品id
	Validationnumber string // 验证工具编号
	T_sn             string // 设备sn
	T_iccid          string // sim卡号
	T_imei           string // 模组imei
	History_iccid    string // 历史sim卡号
	History_imei     string // 历史模组
	T_state          int    // 1-已出库 2-待使用(已入库)  3-维修中 4-已报废
	T_remark         string // 备注r
	T_project        string // 出库项目
	T_uuid           string //用户uuid
	LendUser         string //借出用户
	CreateTime       string //auto_now_add 第一次保存时才设置时间
	UpdateTime       string //auto_now 每次 model 保存时都会对时间自动更新
}

type ValidationToolHistory_R2 struct {
	Id               int      //  合同编号
	T_class          int      //  产品分类 T_class=0 冷链验证 	// 产品id
	Validationnumber string   //  验证工具编号
	T_sn             string   //  设备sn
	T_iccid          string   //  sim卡号
	T_imei           string   //  模组imei
	History_iccid    string   //  历史sim卡号
	History_imei     string   //  历史模组
	T_state          int      //  1-已出库 2-待使用(已入库)  3-维修中 4-已报废
	T_remark         string   //  备注r
	T_project        string   //  出库项目
	T_uuid           string   //	用户uuid
	LendUser         string   //	借出用户
	BatchNumber      string   //	批号
	T_sn_List        []string //	批号
	CreateTime       string   //	auto_now_add 第一次保存时才设置时间
	UpdateTime       string   //	auto_now 每次 model 保存时都会对时间自动更新
}

func ValidationToolHistoryToValidationToolHistory_R(t ValidationToolHistory) (r ValidationToolHistory_R) {
	r.Id = t.Id
	r.T_class = t.T_class
	r.Validationnumber = t.Validationnumber
	r.T_sn = t.T_sn
	r.T_iccid = t.T_iccid
	r.T_imei = t.T_imei
	r.History_iccid = t.History_iccid
	r.History_imei = t.History_imei
	r.T_state = t.T_state
	r.T_remark = t.T_remark
	r.T_project = t.T_project
	r.T_uuid = t.T_uuid
	r.LendUser = t.LendUser
	r.CreateTime = t.CreateTime.Format("2006-01-02 15:04:05")
	r.UpdateTime = t.UpdateTime.Format("2006-01-02 15:04:05")
	return r
}
func ValidationToolHistoryToValidationToolHistory_R2(t ValidationToolHistory) (r ValidationToolHistory_R2) {
	r.Id = t.Id
	r.T_class = t.T_class
	r.Validationnumber = t.Validationnumber
	r.T_sn = t.T_sn
	r.T_iccid = t.T_iccid
	r.T_imei = t.T_imei
	r.History_iccid = t.History_iccid
	r.History_imei = t.History_imei
	r.T_state = t.T_state
	r.T_remark = t.T_remark
	r.T_project = t.T_project
	r.T_uuid = t.T_uuid
	r.LendUser = t.LendUser
	r.BatchNumber = t.BatchNumber
	r.T_sn_List = ReadSNListBytBatchNumber(t.BatchNumber)
	r.CreateTime = t.CreateTime.Format("2006-01-02 15:04:05")
	r.UpdateTime = t.UpdateTime.Format("2006-01-02 15:04:05")
	return r
}

func (t *ValidationToolHistory) TableName() string {
	return "validation_tool_history"
}

type ValidationToolRecordImpl struct {
	orm orm.Ormer
}

func NewValidationToolRecord(orm orm.Ormer) *ValidationToolRecordImpl {
	return &ValidationToolRecordImpl{orm: orm}
}

func init() {
	//注册模型
	orm.RegisterModel(new(ValidationToolHistory))
}

// ADDValidationTool 添加
func (dao *ValidationToolRecordImpl) ADD(r ValidationTool, BatchNumber string) (id int64, err error) {
	his := ValidationToolHistory{
		T_class:          r.T_class,
		Validationnumber: r.Validationnumber,
		T_sn:             r.T_sn,
		T_iccid:          r.T_iccid,
		T_imei:           r.T_imei,
		History_iccid:    r.History_iccid,
		History_imei:     r.History_imei,
		T_state:          r.T_state,
		T_remark:         r.T_remark,
		T_project:        r.T_project,
		T_uuid:           r.T_uuid,
		LendUser:         r.LendUser,
		BatchNumber:      BatchNumber,
	}
	id, err = dao.orm.Insert(&his)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return id, err

}
func ReadSNListBytBatchNumber(BatchNumber string) (r []string) {
	o := orm.NewOrm()
	qs := o.QueryTable(new(ValidationToolHistory))
	var maps []ValidationToolHistory
	_, err := qs.Filter("BatchNumber", BatchNumber).OrderBy("CreateTime").All(&maps)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	for _, v := range maps {
		r = append(r, v.T_sn)
	}
	return
}

func (dao *ValidationToolRecordImpl) Read_Validation_List(Validationnumber, T_sn, T_iccid, T_imei, T_State, LendUser, T_project, T_class string,
	page, page_z int) (r []ValidationToolHistory_R, cnt int64) {
	var offset int
	if page <= 1 {
		offset = 0
	} else {
		offset = (page - 1) * page_z
	}

	// 过滤
	sqlWhere := " WHERE id > 0"
	if len(Validationnumber) > 0 {
		sqlWhere += " AND validationnumber like \"%" + Validationnumber + "%\""
	}
	if len(T_State) > 0 {
		sqlWhere += " AND t_state like \"%" + T_State + "%\""
	}
	if len(T_sn) > 0 {
		sqlWhere += " AND t_sn like \"%" + T_sn + "%\""
	}
	if len(T_iccid) > 0 {
		sqlWhere += " AND t_iccid like \"%" + T_iccid + "%\""
	}
	if len(T_imei) > 0 {
		sqlWhere += " AND t_imei like \"%" + T_imei + "%\""
	}
	if len(LendUser) > 0 {
		sqlWhere += " AND lend_user like \"%" + LendUser + "%\""
	}
	if len(T_project) > 0 {
		sqlWhere += " AND t_project like \"%" + T_project + "%\""
	}
	if len(T_class) > 0 {
		sqlWhere += " AND t_class like \"%" + T_class + "%\""
	}
	var maps_z []orm2.ParamsList
	// 获取总条数
	sql := "SELECT COUNT(*) FROM validation_tool_history"
	sql = sql + sqlWhere
	fmt.Println(sql)
	_, err := dao.orm.Raw(sql).ValuesList(&maps_z)
	if err != nil {
		return r, 0
	}
	if len(maps_z) == 0 {
		return r, 0
	}
	sql = "SELECT * FROM validation_tool_history"

	sql = sql + sqlWhere

	sql += " ORDER BY create_time DESC"

	sql += " LIMIT " + strconv.Itoa(offset) + "," + strconv.Itoa(page_z)

	fmt.Println(sql)
	var maps []ValidationToolHistory
	_, err = dao.orm.Raw(sql).QueryRows(&maps)
	if err != nil {
		return r, 0
	}
	for _, v := range maps {
		r = append(r, ValidationToolHistoryToValidationToolHistory_R(v))
	}

	count, _ := strconv.Atoi(maps_z[0][0].(string))
	return r, int64(count)
}
func (dao *ValidationToolRecordImpl) Read_Operation_List(T_sn, T_State, LendUser, T_project string,
	page, page_z int) (r []ValidationToolHistory_R2, cnt int64) {
	var offset int
	if page <= 1 {
		offset = 0
	} else {
		offset = (page - 1) * page_z
	}

	// 过滤
	sqlWhere := " WHERE id > 0"
	if len(T_State) > 0 {
		sqlWhere += " AND t_state like \"%" + T_State + "%\""
	}
	if len(T_sn) > 0 {
		sqlWhere += " AND t_sn like \"%" + T_sn + "%\""
	}
	if len(LendUser) > 0 {
		sqlWhere += " AND lend_user like \"%" + LendUser + "%\""
	}
	if len(T_project) > 0 {
		sqlWhere += " AND t_project like \"%" + T_project + "%\""
	}
	var maps_z []orm2.ParamsList
	// 获取总条数
	sql := "SELECT COUNT(*) from (select batch_number FROM validation_tool_history "
	sql += sqlWhere
	sql += " GROUP BY batch_number) A"
	fmt.Println(sql)
	_, err := dao.orm.Raw(sql).ValuesList(&maps_z)
	if err != nil {
		return r, 0
	}
	if len(maps_z) == 0 {
		return r, 0
	}
	sql = "SELECT * FROM validation_tool_history"

	sql = sql + sqlWhere

	sql += " GROUP BY batch_number ORDER BY batch_number DESC"

	sql += " LIMIT " + strconv.Itoa(offset) + "," + strconv.Itoa(page_z)

	fmt.Println(sql)
	var maps []ValidationToolHistory
	_, err = dao.orm.Raw(sql).QueryRows(&maps)
	if err != nil {
		return r, 0
	}
	for _, v := range maps {
		r = append(r, ValidationToolHistoryToValidationToolHistory_R2(v))
	}

	count, _ := strconv.Atoi(maps_z[0][0].(string))
	return r, int64(count)
}
